package info.textgrid.lab.collatex.ui.utils;

import info.textgrid.lab.core.swtutils.AdapterUtils;

import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.PlatformUI;

import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

/**
 * <p>
 * Helper that asks the user to save unsaved editors.
 * </p>
 * 
 * Clients can {@linkplain #addFilter(Predicate) add filters} to ask only for
 * selected editors.
 */
public class SaveConfirmation extends MessageDialog {

	private final Predicate<IEditorReference> IS_DIRTY = new Predicate<IEditorReference>() {

		@Override
		public boolean apply(final IEditorReference input) {
			return input.isDirty();
		}
	};


	@SuppressWarnings("unchecked")
	private final List<Predicate<IEditorReference>> filters = Lists
	.newArrayList(IS_DIRTY);

	private CheckboxTableViewer viewer;

	private IEditorReference[] checkedElements = new IEditorReference[0];

	/**
	 * Creates a new save confirmation.
	 * 
	 * @param parentShell
	 *            The parent window for the dialog
	 * @param dialogMessage
	 *            The message to display above the list of open editors
	 */
	public SaveConfirmation(final Shell parentShell,
			final String dialogMessage) {
		this(parentShell, Messages.SaveConfirmation_SaveConfirmationTitle, null, dialogMessage,
				MessageDialog.QUESTION_WITH_CANCEL, new String[] {
				Messages.SaveConfirmation_SaveSelectedResources, Messages.SaveConfirmation_ContinueAnyway,
				Messages.SaveConfirmation_Cancel }, 0);
	}

	/**
	 * Adds a filter that decides whether a specific editor reference should be
	 * handled by this dialog or not. Multiple filters can be added, only
	 * editors which are dirty and to which <em>all</em> configured filters
	 * apply are handled by this dialog.
	 * 
	 * @return this dialog class, use this for chaining.
	 */
	public SaveConfirmation addFilter(final Predicate<IEditorReference> filter) {
		filters.add(filter);
		return this;
	}


	private SaveConfirmation(final Shell parentShell, final String dialogTitle,
			final Image dialogTitleImage, final String dialogMessage,
			final int dialogImageType, final String[] dialogButtonLabels,
			final int defaultIndex) {
		super(parentShell, dialogTitle, dialogTitleImage, dialogMessage,
				dialogImageType, dialogButtonLabels, defaultIndex);
	}

	@Override
	protected Control createCustomArea(final Composite parent) {
		viewer = CheckboxTableViewer
				.newCheckList(parent, SWT.FULL_SELECTION);
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(new LabelProvider() {

			@Override
			public Image getImage(final Object element) {
				if (element instanceof IEditorReference)
					return ((IEditorReference) element).getTitleImage();
				return null;
			}

			@Override
			public String getText(final Object element) {
				if (element instanceof IEditorReference)
					return ((IEditorReference) element).getPartName();
				return element.toString();
			}

		});
		viewer.getControl().setLayoutData(
				GridDataFactory.fillDefaults().grab(true, false).create());

		final Iterable<IEditorReference> askFor = getEditorsToConfirm();
		final IEditorReference[] references = Iterables.toArray(askFor,
				IEditorReference.class);
		viewer.setInput(references);
		viewer.setCheckedElements(references);

		return viewer.getControl();
	}

	@Override
	protected void buttonPressed(final int buttonId) {
		checkedElements = AdapterUtils.getAdapters(viewer.getCheckedElements(),
				IEditorReference.class, false);
		super.buttonPressed(buttonId);
	}

	private Iterable<IEditorReference> getEditorsToConfirm() {
		return Iterables.filter(
				Arrays.asList(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getActivePage()
						.getEditorReferences()), Predicates.and(filters));
	}

	/**
	 * Asks the user whether the editors are to be saved.
	 * 
	 * <p>
	 * This method first calculates which editors are to be handled by the
	 * dialog by retrieving all open editors that are
	 * {@linkplain ISaveablePart#isDirty() dirty} and pass the
	 * {@linkplain #addFilter(Predicate) filters}. When this list is empty, no
	 * dialog is shown and <code>true</code> is returned.
	 * </p>
	 * <p>
	 * Otherwise, we present the user a checkable list of editors to be saved.
	 * The user can either cancel the operation—in this case {@link #ask()}
	 * returns <code>false</code>. Or the user can select to continue, in this
	 * case the method saves all editors from which the user has not removed the
	 * check mark in the dialog and returns <code>true</code>.
	 * 
	 * @return <code>true</code> to continue, <code>false</code> to cancel. Note
	 *         that a return value of <code>true</code> does <em>not</em>
	 *         guarantee that all dirty editors have been saved.
	 */
	public boolean ask() {
		final Iterable<IEditorReference> toConfirm = getEditorsToConfirm();
		if (toConfirm.iterator().hasNext()) {
			final int result = open();
			switch (result) {
			case 0:
				saveSelectedEditors(null);
				return true;
			case 1:
				return true;
			case 2:
				return false;
			}
		}
		return true;
	}

	private void saveSelectedEditors(final IProgressMonitor monitor) {
		final SubMonitor progress = SubMonitor.convert(monitor,
				Messages.SaveConfirmation_SavingOpenEditors,
				checkedElements.length * 10);
		for (final IEditorReference o : checkedElements)
			o.getEditor(true).doSave(progress.newChild(10));
	}
}